// $Id: parent.js,v 1.1.2.3 2009/05/27 11:15:56 markuspetrux Exp $

/**
 * Modal Frame object for parent windows.
 */
Drupal.modalFrame = Drupal.modalFrame || {
  dirtyFormsWarning: Drupal.t('Your changes will be lost if you close this popup now.'),
  options: {},
  iframe: { $container: null, $element: null },
  isOpen: false
};

/**
 * Open the popup dialog.
 */
Drupal.modalFrame.open = function(options) {
  var self = this;

  // Just one modal is allowed.
  if (self.isOpen || $('#modalframe-container').size()) {
    return false;
  }

  // Build modal frame options structure.
  self.options = {
    url: options.url,
    size: self.sanitizeSize({width: options.width, height: options.height}),
    autoFit: (options.autoFit == undefined || options.autoFit ? true : false),
    autoResize: (options.autoResize ? true : false),
    onSubmit: options.onSubmit
  };

  // Create the dialog and related DOM elements.
  self.create(options);

  // Prepare the dialog with the "loading" behavior.
  $('.modalframe .ui-dialog-content').addClass('modalframe-loading');
  $('.modalframe .ui-dialog-title').html(Drupal.t('Loading...'));

  // Open the dialog offscreen where we can set its size.
  self.iframe.$container
    .dialog('option', {position: ['-999em', '-999em']})
    .dialog('open')
    .dialog('option', {width: self.options.size.width, height: self.options.size.height});

  // Adjust size of the iframe container and element.
  self.iframe.$container.width(self.options.size.width).height(self.options.size.height - $('.modalframe .ui-dialog-titlebar').outerHeight(true));
  self.iframe.$element.width(self.iframe.$container.innerWidth()).height(self.iframe.$container.innerHeight());
  self.iframe.$element.hide();

  // Hide the dialog, move it to center of screen and then fade it in.
  $('.modalframe').hide();
  self.iframe.$container.dialog('option', {position: ['center', 'middle']});
  $('.modalframe').fadeIn('slow', function() {
    // Load the document on hidden iframe (see bindChild method).
    self.load(self.options.url);
  });

  return true;
};

/**
 * Create the popup dialog.
 */
Drupal.modalFrame.create = function() {
  var self = this;

  // Note: We use scrolling="yes" for IE as a workaround to yet another IE bug
  // where the horizontal scrollbar is always rendered no matter how wide the
  // iframe element is defined.
  self.iframe.$element = $('<iframe id="modalframe-element" name="modalframe-element"'+ ($.browser.msie ? ' scrolling="yes"' : '') +'/>');
  self.iframe.$container = $('<div id="modalframe-container"/>').append(self.iframe.$element);
  $('body').append(self.iframe.$container);

  self.iframe.$container.dialog({
    modal: true,
    autoOpen: false,
    closeOnEscape: true,
    resizable: false,
    title: Drupal.t('Loading...'),
    dialogClass: 'modalframe',
    open: function() {
      $('.modalframe .ui-dialog-titlebar-close:not(.modalframe-processed)').addClass('modalframe-processed')
        .attr('href', 'javascript:void(0)')
        .attr('title', Drupal.t('Close'))
        .unbind('click').bind('click', function() { try { self.close(false); } catch(e) {}; return false; });
      if (self.options.autoResize) {
        // @fixme: the autoResize feature does not seem to work very well.
        $(window).bind('resize.modalframe', function() {
          if (self.isOpen && self.isObject(self.iframe.childWindowSize)) {
            self.resize(self.iframe.childWindowSize, true);
          }
        });
      }
      self.fixPosition($('.modalframe'), true);
      self.isOpen = true;
    },
    beforeclose: function() {
      if (self.beforeCloseEnabled) {
        return true;
      }
      if (!self.beforeCloseIsBusy) {
        self.beforeCloseIsBusy = true;
        setTimeout(function() { self.close(false); }, 1);
      }
      return false;
    },
    close: function() {
      if (self.options.autoResize) {
        $(window).unbind('resize.modalframe');
      }
      $('.modalframe .ui-dialog-titlebar-close').unbind('keypress.modalframe').unbind('keydown.modalframe');
      self.fixPosition($('.modalframe'), false);
      self.iframe.childWindowSize = null;
      self.iframe.document = null;
      self.iframe.Drupal = null;
      self.iframe.$element.remove();
      self.iframe.$element = null;
      self.iframe.$container.dialog('destroy').remove();
      self.iframe.$container = null;
      if (self.beforeCloseEnabled) {
        delete self.beforeCloseEnabled;
      }
      if (self.beforeCloseIsBusy) {
        delete self.beforeCloseIsBusy;
      }
      self.isOpen = false;
    }
  });
};

/**
 * Load the given URL into the dialog iframe.
 */
Drupal.modalFrame.load = function(url) {
  var self = this;
  var iframe = self.iframe.$element.get(0);
  // Get the window object of the iframe window.
  // @see http://xkr.us/articles/dom/iframe-document/
  var doc = (iframe.contentWindow || iframe.contentDocument);
  if (doc.document) {
    doc = doc.document;
  }
  doc.location.replace(url);
};

/**
 * Check if the dialog can be closed.
 */
Drupal.modalFrame.canClose = function() {
  var self = this;
  if (!self.isOpen) {
    return false;
  }
  if (self.isObject(self.iframe.Drupal)) {
    // Prompt user for confirmation to close dialog if children window has
    // dirty forms.
    if (self.isObject(self.iframe.Drupal.dirtyForms)) {
      if (self.iframe.Drupal.dirtyForms.isDirty() && !confirm(self.dirtyFormsWarning)) {
        return false;
      }
      self.iframe.Drupal.dirtyForms.warning = null;
    }
    // Disable onBeforeUnload behaviors on child window.
    if (self.isObject(self.iframe.Drupal.onBeforeUnload)) {
      self.iframe.Drupal.onBeforeUnload.disable();
    }
  }
  return true;
};

/**
 * Close event handler for the popup dialog.
 */
Drupal.modalFrame.close = function(args) {
  var self = this;

  // Check if the dialog can be closed.
  if (!self.canClose()) {
    delete self.beforeCloseIsBusy;
    return false;
  }

  // Hide and destroy the dialog.
  function closeDialog() {
    self.beforeCloseEnabled = true;
    self.iframe.$container.dialog('close');
    if ($.isFunction(self.options.onSubmit)) {
      self.options.onSubmit(args);
    }
  }
  if (!self.isObject(self.iframe.$element) || !self.iframe.$element.size() || !self.iframe.$element.is(':visible')) {
    closeDialog();
  }
  else {
    self.iframe.$element.fadeOut('fast', function() {
      // Using jQuery UI 1.7.1 - Animation of dialog height breaks IE.
      var animationOptions = ($.browser.msie ? {opacity: 'hide'} : {height: 'hide', opacity: 'hide'});
      $('.modalframe').animate(animationOptions, closeDialog);
    });
  }
  return true;
};

/**
 * Bind the child window.
 */
Drupal.modalFrame.bindChild = function(iFrameWindow, isClosing) {
  var self = this;
  self.iframe.document = iFrameWindow.document;
  self.iframe.Drupal = iFrameWindow.Drupal;

  // We are done if the child window is closing.
  if (isClosing) {
    return;
  }

  // Update the dirty forms warning on the child window.
  if (self.isObject(self.iframe.Drupal.dirtyForms)) {
    self.iframe.Drupal.dirtyForms.warning = self.dirtyFormsWarning;
  }

  // Update the dialog title with the child window title.
  $('.modalframe .ui-dialog-title').html($(self.iframe.document).attr('title'));

  // Perform animation to show the iframe element using a separate thread.
  setTimeout(function() {
    self.iframe.$element.fadeIn('slow', function() {
      // Adjust modal dialog to fit the iframe content?
      if (self.options.autoFit) {
        self.iframe.childWindowSize = {
          width: iFrameWindow.$(self.iframe.document).width(),
          height: iFrameWindow.$(self.iframe.document).height()
        };
        self.resize(self.iframe.childWindowSize, false);
      }

      // Set focus to the first tabbable element in the content area or the
      // first button. If there are no tabbable elements, set focus on the
      // close button of the dialog itself.
      // Logic based on open() method in ui.dialog.js
      // @fixme: Tabulation on IE does not seem to work very well. :(

      var $first, $closeButton = $('.modalframe .ui-dialog-titlebar-close');
      // Prevent browser security context related errors. Not sure why, but
      // this happened to me with IE6 on my development environment.
      try { $first = $(':tabbable:first', self.iframe.document); } catch(e) {};
      if (!$first || !$first.size()) {
        $closeButton.focus();
      }
      else {
        try { $first.focus(); } catch(e) {};

        // Prevent tabbing out of the child window, except for the close button
        // of the dialog itself.
        var $childTabbables = $(':tabbable', self.iframe.document),
            first = $childTabbables.filter(':first')[0],
            last  = $childTabbables.filter(':last')[0];
        if ($childTabbables.size()) {
          $closeButton.bind('keypress.modalframe', function(event) {
            if (event.keyCode == $.ui.keyCode.TAB) {
              setTimeout(function() { (event.shiftKey ? last : first).focus(); }, 1);
              return false;
            }
          });
          $(self.iframe.document).bind('keypress.modalframe', function(event) {
            if (event.keyCode == $.ui.keyCode.TAB) {
              if ((event.target == last && !event.shiftKey) || (event.target == first && event.shiftKey)) {
                setTimeout(function() { $closeButton.focus(); }, 1);
              }
            }
          }).bind('keydown.modalframe', function(event) {
            if (event.keyCode && event.keyCode == $.ui.keyCode.ESCAPE) {
              setTimeout(function() { self.close(false); }, 1);
            }
          });
        }
      }
    });
  }, 1);
};

/**
 * Unbind the child window.
 */
Drupal.modalFrame.unbindChild = function(iFrameWindow) {
  var self = this;
  // Unbind keyboard event handlers.
  try {
    $('.modalframe .ui-dialog-titlebar-close').unbind('keypress.modalframe');
    $(iFrameWindow.document).unbind('keypress.modalframe').unbind('keydown.modalframe');
  } catch(e) {};
  // Change the modal dialog title.
  $('.modalframe .ui-dialog-title').html(Drupal.t('Please, wait...'));
  // Hide the iframe element.
  self.iframe.$element.fadeOut('fast');
};

/**
 * Check if the given variable is an object.
 */
Drupal.modalFrame.isObject = function(x) {
  return (x !== null && typeof x === 'object');
};

/**
 * Fix the position of the popup window.
 *
 * Possible alternative to position:'fixed' for IE6:
 * @see http://www.howtocreate.co.uk/fixedPosition.html
 */
Drupal.modalFrame.fixPosition = function($element, isOpen) {
  var $window = $(window);
  if ($.browser.msie && parseInt($.browser.version) <= 6) {
    // IE6 does not support position:'fixed'.
    // Lock the window scrollBar instead.
    if (isOpen) {
      var yPos = $window.scrollTop();
      var xPos = $window.scrollLeft();
      $window.bind('scroll.modalframe', function() {
        window.scrollTo(xPos, yPos);
        // Default browser action cannot be prevented here.
      });
    }
    else {
      $window.unbind('scroll.modalframe');
    }
  }
  else {
    // Use CSS to do it on other browsers.
    if (isOpen) {
      var offset = $element.offset();
      $element.css({
        left: (offset.left - $window.scrollLeft()),
        top: (offset.top - $window.scrollTop()),
        position: 'fixed'
      });
    }
  }
};

/**
 * Sanitize dialog size.
 */
Drupal.modalFrame.sanitizeSize = function(size) {
  var width, height;
  var $window = $(window);
  var minWidth = 300, maxWidth = parseInt($window.width() * .85);
  if (typeof size.width != 'number') {
    width = maxWidth;
  }
  else if (size.width < minWidth || size.width > maxWidth) {
    width = Math.min(maxWidth, Math.max(minWidth, size.width));
  }
  else {
    width = size.width;
  }
  var minHeight = 100, maxHeight = parseInt($window.height() * .85);
  if (typeof size.height != 'number') {
    height = maxHeight;
  }
  else if (size.height < minHeight || size.height > maxHeight) {
    height = Math.min(maxHeight, Math.max(minHeight, size.height));
  }
  else {
    height = size.height;
  }
  return {width: width, height: height};
};

/**
 * Resize modal dialog.
 */
Drupal.modalFrame.resize = function(size, hideFrameWhileAnimating) {
  var self = this;

  // Compute frame size.
  var frameSize = self.sanitizeSize(size);
  // IE seems to need a bit more height here.
  if ($.browser.msie) {
    frameSize.height += 20;
  }

  // Compute dialog size and position centered on viewport.
  var $window = $(window), $document = $(document);
  var dialogSize = frameSize, minTop = $document.scrollTop();

  dialogSize.height += $('.modalframe .ui-dialog-titlebar').outerHeight(true);

  var animationOptions = {
    width: dialogSize.width,
    height: dialogSize.height,
    left: $document.scrollLeft() + (($window.width() - dialogSize.width) / 2),
    top: minTop + Math.max(minTop, (($window.height() - dialogSize.height) / 2))
  };

  // Hide the frame and perform the animation.
  if (hideFrameWhileAnimating) {
    self.iframe.$element.css({visibility: 'hidden'});
  }
  $('.modalframe').animate(animationOptions, 'fast', function() {
    setTimeout(function() {
      // Resize and show the frame.
      self.iframe.$element
        .width(frameSize.width)
        .height(frameSize.height)
        .css({visibility: 'visible'});
      // Update the dialog size so that UI internals are aware of the change.
      self.iframe.$container.dialog('option', {
        width: dialogSize.width,
        height: dialogSize.height
      });
    }, 1);
  });
};
